package MT::Test::Selenium;

use Role::Tiny::With;
use strict;
use warnings;
use JSON::PP ();    # silence redefine warnings
use JSON;
use Test::More;
use Test::TCP;
use Plack::Runner;
use Plack::Builder;
use Plack::App::Directory;
use File::Spec;
use File::Which qw/which/;
use URI;
use URI::QueryParam;
use MT::PSGI;
use constant DEBUG => $ENV{MT_TEST_SELENIUM_DEBUG} ? 1 : 0;
use constant MY_HOST => $ENV{TRAVIS} ? $ENV{HOSTNAME} : '127.0.0.1';

with qw(
    MT::Test::Role::Wight
    MT::Test::Role::Request
    MT::Test::Role::WebQuery
);

our %EXTRA = (
    "Selenium::Chrome" => {
        'goog:chromeOptions' => {
            args => [
                'headless', ( DEBUG ? ('enable-logging') : () ),
                'window-size=1280,800', 'no-sandbox',
            ],
            perfLoggingPrefs => {},
        },
        loggingPrefs => { performance => 'ALL' },
        binaries     => [
            'chromedriver',
            '/usr/bin/chromedriver',
            '/usr/lib/chromium-browser/chromedriver',
            '/usr/lib64/chromium-browser/chromedriver',
        ],
    },
    "Selenium::Remote::Driver" => {
        'goog:chromeOptions' => {
            args => [
                'headless', ( DEBUG ? 'enable-logging' : () ),
                'window-size=1280,800', 'no-sandbox',
            ],
        },
        travis => {
            remote_server_addr => 'chromedriver',
            port               => 9515,
        },
    },
);

sub new {
    my ( $class, $env ) = @_;

    my $driver_class = $ENV{MT_TEST_SELENIUM_DRIVER}
        || ( $ENV{TRAVIS} ? 'Selenium::Remote::Driver' : 'Selenium::Chrome' );
    eval "require $driver_class" or plan skip_all => "No $driver_class";

    $Selenium::Remote::Driver::FORCE_WD2 = 1;

    my $extra = $EXTRA{$driver_class} || {};

    my %driver_opts = (
        auto_close          => 1,
        default_finder      => 'css',
        extra_capabilities  => $extra,
        acceptInsecureCerts => 1,
        timeout             => 10,
    );
    for my $binary ( @{ delete $extra->{binaries} || [] } ) {
        $binary = _fix_binary($binary) or next;
        $driver_opts{binary} = $binary;
        last;
    }

    my $travis_config = delete $extra->{travis};
    if ( $ENV{TRAVIS} ) {
        %driver_opts = ( %driver_opts, %$travis_config );
    }

    if (DEBUG) {
        my $log_file = "$ENV{MT_HOME}/selenium_log.txt";
        $driver_opts{custom_args} = "--log-path=$log_file --verbose";
    }

    my $driver = eval { $driver_class->new(%driver_opts) }
        or plan skip_all => "Failed to instantiate $driver_class: $@";

    $driver->debug_on if DEBUG;

    my $server = Test::TCP->new(
        code => sub {
            my $port = shift;

            my $host  = MY_HOST;
            my %extra = ( CGIPath => "http://$host:$port/cgi-bin/" );
            $env->update_config(%extra);

            my $app        = MT::PSGI->new->to_app;
            my $static_app = Plack::App::Directory->new(
                root => "$ENV{MT_HOME}/mt-static" );

            my $builder = builder {
                mount "/mt-static" => builder {
                    enable 'AccessLog' if DEBUG;
                    $static_app;
                };
                mount "/" => builder {
                    enable 'AccessLog' if DEBUG;
                    $app;
                };
            };
            my $runner = Plack::Runner->new( app => $builder );
            $runner->parse_options(
                '--port'   => $port,
                '--env'    => ( DEBUG ? 'development' : 'test' ),
                '--server' => 'Standalone',
            );
            $runner->run;
        },
    );

    my $port     = $server->port;
    my $host     = MY_HOST;
    my $base_url = URI->new("http://$host:$port");

    bless {
        server   => $server,
        base_url => $base_url,
        driver   => $driver,
        pid      => $$,
    }, $class;
}

sub _fix_binary {
    my $binary = shift;
    if ( File::Spec->file_name_is_absolute($binary) ) {
        return $binary if -e $binary && -x _;
    }
    else {
        which($binary);
    }
}

sub driver { shift->{driver} }

sub DESTROY {
    my $self = shift;
    return unless $self->{pid} eq $$;
    my $driver = $self->{driver} or return;
    $driver->quit;
}

sub base_url {
    my $self = shift;
    $self->{base_url}->clone;
}

sub login {
    my ( $self, $user ) = @_;
    my $url = $self->base_url;
    $url->path('/cgi-bin/mt.cgi');
    $self->driver->get("$url");
    $self->{content} = $self->driver->get_page_source;
    my $form = $self->form;
    $form->param( username => $user->name );
    $form->param( password => $user->id == 1 ? 'Nelson' : 'pass' );

    $self->_post_form($form);
}

sub _post_form {
    my ( $self, $form ) = @_;
    my $submit;
    for my $input ( $form->inputs ) {
        my $type = $input->type;
        if ( $type =~ /(?:text|password)/ ) {
            my $elem = $self->_find_by_input($input);
            $elem->clear;
            $elem->send_keys( $input->value );
        }
        if ( $type eq 'submit' ) {
            $submit = $self->_find_by_input($input);
        }
    }
    $submit->click();
}

sub _find_by_input {
    my ( $self, $input ) = @_;
    if ( $input->id ) {
        return $self->driver->find_element_by_id( $input->id );
    }
    elsif ( $input->name ) {
        my $type = $input->type;
        if ( $type eq 'select' ) {
            Carp::croak "not implemented";
        }
        elsif ( $type eq 'textarea' ) {
            Carp::croak "not implemented";
        }
        else {
            return $self->driver->find_element(
                'input[name=' . $input->name . ']' );
        }
    }
    Carp::croak "Can't find elem from input";
}

sub mt_url {
    my ( $self, $params ) = @_;
    my $url = $self->base_url;
    $url->path("/cgi-bin/mt.cgi");
    $url->query_form_hash($params) if $params;
    $url;
}

sub request {
    my ( $self, $params ) = @_;
    my $method = delete $params->{__request_method};
    my $request_url;
    if ( $method eq 'GET' ) {
        $request_url = $self->mt_url($params);
        $self->driver->get("$request_url");
        $self->{content} = $self->driver->get_page_source;
    }
    elsif ( $method eq 'POST' ) {
        $request_url = $self->mt_url;
        my $submit;
        for my $key ( keys %$params ) {
            my $input = eval { $self->driver->find_element_by_name($key) };
            if ($input) {
                my $tag = lc $input->get_tag_name;
                if ( !$input->is_enabled or $input->is_hidden ) {
                    next;
                }
                if ( $tag eq 'input' ) {
                    my $type = lc $input->get_attribute('type');
                    if ( $type eq 'submit' ) {
                        $submit = $input;
                    }
                    elsif ( $type eq 'checkbox' ) {
                    }
                    elsif ( $type eq 'radio' ) {
                    }
                    else {
                        $input->clear if defined $input->get_value;
                        $input->send_keys( $params->{$key} );
                    }
                }
                elsif ( $tag eq 'select' ) {
                    $input->click;
                    for my $option ( @{ $input->children('option') || [] } ) {
                        if ( $option->get_value eq $params->{$key} ) {
                            $option->set_selected;
                        }
                        elsif ( $option->is_selected ) {
                            $option->toggle;
                        }
                    }
                }
            }
        }
        my $source = $self->driver->get_page_source;
        if ($submit) {
            $submit->click;
        }
        else {
            $submit = $self->driver->find_element_by_class('btn-primary');
            $submit->click;
        }
    }

    if (DEBUG) {
        ## Typically warnings from jquery-migrate.js
        my $console_log = $self->driver->get_log('browser');
        note explain $console_log if @{ $console_log || [] };
    }

    my $logs   = $self->_get_response_logs;
    my @errors = grep { $_->{status} !~ /^[23]/ } @$logs;
    if (@errors) {
        note explain \@errors;
    }
    my $res;
    for my $log (@$logs) {
        next unless $log->{url} eq $request_url;

        # TODO: take care of redirection
        $res = HTTP::Response->new( $log->{status}, $log->{statusText},
            [ %{ $log->{headers} || {} } ] );
        $res->content( $self->{content} );
    }
    $res ||= HTTP::Response->new(200);
    return $self->{res} = $res;
}

sub _get_request_logs {
    my $self = shift;
    my $logs = $self->driver->get_log('performance');
    my @requests;
    for my $log (@$logs) {
        next unless $log->{message} =~ /Network\.requestWillBeSent/;
        my $message = decode_json( $log->{message} );
        my $req     = $message->{message}{params}{request} or next;
        push @requests, { map { $_ => $req->{$_} } qw/headers method url/ };
    }
    return \@requests;
}

sub _get_response_logs {
    my $self = shift;
    my $logs = $self->driver->get_log('performance');
    my @responses;
    for my $log (@$logs) {
        next unless $log->{message} =~ /Network\.responseReceived/;
        my $message = decode_json( $log->{message} );
        my $res     = $message->{message}{params}{response} or next;
        ## Add timing if requested?
        push @responses,
            { map { $_ => $res->{$_} }
                qw/headers status statusText url requestHeaders/ };
    }
    return \@responses;
}

1;
